#version 430 core

in VertexData {
    vec3 position_world;
    vec3 normal_world;
    vec2 uv;
} vert;

layout(location = 0) out vec4 color;
layout(location = 1) out vec4 normal;
layout(location = 2) out vec4 depth;

uniform vec3 camera_world;
uniform float farP;

uniform sampler2D diffuse;
uniform samplerCube depthMap;
uniform samplerCube depthMap2;

uniform float constantBias;
uniform float slopeBias;

uniform bool isObstacle;
uniform bool isLava;

uniform struct DirectionalLight {
    vec3 color;
    vec3 direction;
} dirL;

uniform struct PointLight {
    vec3 color;
    vec3 position;
    vec3 attenuation;
} pointL;

float near = 0.1f;
float far = 1000.0f;
const float levels = 4.0;

vec3 cel(vec3 n, vec3 l, vec3 diffuseC) {
    float d = length(l) * 0.5;
    l = normalize(l);
    vec3 r = reflect(-l, n);
    float diffuseF = max(0, dot(n, l));
    float quantizedDiffuse = floor(diffuseF * levels) / levels;
    return quantizedDiffuse * diffuseC;
}

float linearizeDepth(float depth) {
    return (2.0 * near * far) / (far + near - (depth * 2.0 - 1.0) * (far - near));
}

vec3 poissonDisk[4] = vec3[](
  vec3( -0.94201624, -0.39906216, -0.39906216 ),
  vec3( 0.94558609, -0.76890725, 0.39906216 ),
  vec3( -0.094184101, -0.92938870, -0.39906216 ),
  vec3( 0.34495938, 0.29387760, 0.39906216 )
);

float calcShadow() {
    vec3 fragPos = vert.position_world;

    vec3 fragToLight = vec3(0.0);
    
    fragToLight = fragPos - pointL.position;    

    float closestDepth = 0.0;   

    closestDepth = texture(depthMap, fragToLight).r;   
 
    closestDepth *= farP;
    
    float currentDepth = length(fragToLight);
 
    float shadow  = 0.0;
    float bias    = 0.15; 
    float samples = 4.0;
    float offset  = 0.005;
    for(float x = -offset; x < offset; x += offset / (samples * 0.5))
    {
        for(float y = -offset; y < offset; y += offset / (samples * 0.5))
        {
            for(float z = -offset; z < offset; z += offset / (samples * 0.5))
            {
                float closestDepth = texture(depthMap, fragToLight + vec3(x, y, z)).r; 
                closestDepth *= farP;
                if(currentDepth - bias > closestDepth)
                    shadow += 1.0;
            }
        }
    }
    shadow /= (samples * samples * samples);
        return shadow;
}

vec3 ChessboardPattern(float noiseScale) {
    float size = 15.0f;

    vec2 uv = vert.uv * size;

    // Chessboard pattern
    bool isEven = (mod(floor(uv.x) + floor(uv.y), 2) == 0);
    vec3 baseColor = isEven ? vec3(1.0f, 1.0f, 0.0f) : vec3(0.1f);
    float noise;
    // Add noise to black cells
    if (!isEven) {
        // Generate noise with sinusoidal pattern
        noise = sin(uv.x * 50.0f) * sin(uv.y * 50.0f);

        // Scale and shift noise values
        noise = noise * 0.1f + 0.1f;

        baseColor += vec3(noise);
    }

    

    if (isLava) {
	uv = vert.uv;

	// Dunkelorangene Farbe
	baseColor = vec3(1.0f, 0.5f, 0.0f);

	// Noise hinzufügen
	noise = 0.0f;
	noise = sin(uv.x * 500.0f * noiseScale) * sin(uv.y * 500.0f * noiseScale);
	noise = noise * 0.2f + 0.2f; // Scale und Shift des Noises

	baseColor += vec3(noise);

    }
    vec3 color = baseColor;

    return color;
}

void main() {
    vec3 n = normalize(vert.normal_world);
    vec3 v = normalize(camera_world - vert.position_world);

    vec4 texColor = texture(diffuse, vert.uv);
    
    vec3 colorCheckerboard = ChessboardPattern(0.2f);

    float shadow1 = calcShadow() * 0.5;  
    
    color = texColor;

    // add directional light contribution
    color.rgb += cel(n, -dirL.direction, dirL.color * texColor.rgb);
    
    // add point light contribution
    color.rgb += cel(n, pointL.position - vert.position_world, pointL.color * texColor.rgb);

    vec3 lighting = (1 - shadow1) * color.rgb;

    if (isObstacle) {
        color = vec4(lighting * colorCheckerboard, 1.0);    
    }
    if (!isObstacle) {
        color = vec4(lighting, 1.0);   
    }
    

    float x = vert.normal_world.x;
    float y = vert.normal_world.y;
    float z = vert.normal_world.z;
    if (x < 0.0f) {x = x * -1.0f;}
    if (y < 0.0f) {y = y * -1.0f;}
    if (z < 0.0f) {z = z * -1.0f;}
    normal = vec4(x, y, z, 1.0);

    float near = 0.1f;
    float far = 100.0f;
   
    depth = vec4(vec3(linearizeDepth(gl_FragCoord.z) / far), 1.0f);
}